@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -65,7 +65,7 @@ bool __uvm_record_lock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t
 {
     bool correct = true;
     uvm_lock_order_t conflicting_order;
-    uvm_thread_context_t *uvm_context = uvm_thread_context_retain();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
     uvm_lock_flags_t mode_flags = (flags & UVM_LOCK_FLAGS_MODE_MASK);
     bool trylock = (flags & UVM_LOCK_FLAGS_TRYLOCK);
 
@@ -124,10 +124,11 @@ bool __uvm_record_lock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t
     }
 
     __set_bit(lock_order, uvm_context->acquired_lock_orders);
+
     if (mode_flags == UVM_LOCK_FLAGS_MODE_EXCLUSIVE)
         __set_bit(lock_order, uvm_context->exclusive_acquired_lock_orders);
 
-    uvm_context->acquired_locks[lock_order] = lock;
+    uvm_context->acquired[lock_order] = lock;
 
     return correct;
 }
@@ -135,7 +136,7 @@ bool __uvm_record_lock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t
 bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t flags)
 {
     bool correct = true;
-    uvm_thread_context_t *uvm_context = uvm_thread_context();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
     uvm_lock_flags_t mode_flags = (flags & UVM_LOCK_FLAGS_MODE_MASK);
     bool exclusive = (mode_flags == UVM_LOCK_FLAGS_MODE_EXCLUSIVE);
     bool out_of_order = (flags & UVM_LOCK_FLAGS_OUT_OF_ORDER);
@@ -149,12 +150,11 @@ bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags
     }
 
     if (uvm_context->skip_lock_tracking > 0)
-        goto done;
+        return true;
 
     if (lock_order == UVM_LOCK_ORDER_INVALID) {
-        correct = false;
         UVM_ERR_PRINT("Releasing a lock (0x%llx) with an invalid lock order\n", (NvU64)lock);
-        goto done;
+        return false;
     }
 
     // Releasing a lock out of order is not incorrect, but often points to
@@ -177,11 +177,11 @@ bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags
         correct = false;
         UVM_ERR_PRINT("Releasing lock %s that's not held\n", uvm_lock_order_to_string(lock_order));
     }
-    else if (uvm_context->acquired_locks[lock_order] != lock) {
+    else if (uvm_context->acquired[lock_order] != lock) {
         correct = false;
         UVM_ERR_PRINT("Releasing a different instance of lock %s than held, held 0x%llx releasing 0x%llx\n",
                       uvm_lock_order_to_string(lock_order),
-                      (NvU64)uvm_context->acquired_locks[lock_order],
+                      (NvU64)uvm_context->acquired[lock_order],
                       (NvU64)lock);
     }
     else if (!!__test_and_clear_bit(lock_order, uvm_context->exclusive_acquired_lock_orders) != exclusive) {
@@ -190,17 +190,14 @@ bool __uvm_record_unlock(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags
                       uvm_lock_order_to_string(lock_order),
                       exclusive ? "exclusive" : "shared", exclusive ? "shared" : "exclusive");
     }
-    uvm_context->acquired_locks[lock_order] = NULL;
-
-done:
-    uvm_thread_context_release();
+    uvm_context->acquired[lock_order] = NULL;
 
     return correct;
 }
 
 bool __uvm_record_downgrade(void *lock, uvm_lock_order_t lock_order)
 {
-    uvm_thread_context_t *uvm_context = uvm_thread_context();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
 
     if (!uvm_context) {
         UVM_ERR_PRINT("Failed to acquire the thread context when recording downgrade of %s\n",
@@ -223,7 +220,7 @@ bool __uvm_record_downgrade(void *lock, uvm_lock_order_t lock_order)
 
 bool __uvm_check_locked(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_t flags)
 {
-    uvm_thread_context_t *uvm_context = uvm_thread_context();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
     uvm_lock_flags_t mode_flags = (flags & UVM_LOCK_FLAGS_MODE_MASK);
     bool exclusive = (mode_flags == UVM_LOCK_FLAGS_MODE_EXCLUSIVE);
 
@@ -240,11 +237,11 @@ bool __uvm_check_locked(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_
         UVM_ERR_PRINT("No lock with order %s acquired at all\n", uvm_lock_order_to_string(lock_order));
         return false;
     }
-    if (uvm_context->acquired_locks[lock_order] != lock) {
+    if (uvm_context->acquired[lock_order] != lock) {
         UVM_ERR_PRINT("Different instance of lock %s acquired, 0x%llx != 0x%llx\n",
                       uvm_lock_order_to_string(lock_order),
                       (NvU64)lock,
-                      (NvU64)uvm_context->acquired_locks[lock_order]);
+                      (NvU64)uvm_context->acquired[lock_order]);
         return false;
     }
 
@@ -261,13 +258,13 @@ bool __uvm_check_locked(void *lock, uvm_lock_order_t lock_order, uvm_lock_flags_
 
 bool __uvm_locking_initialized(void)
 {
-    return uvm_thread_context_initialized();
+    return uvm_thread_context_global_initialized();
 }
 
 bool __uvm_check_lockable_order(uvm_lock_order_t lock_order, uvm_lock_flags_t flags)
 {
     uvm_lock_order_t conflicting_order;
-    uvm_thread_context_t *uvm_context = uvm_thread_context();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
     bool trylock = (flags & UVM_LOCK_FLAGS_TRYLOCK);
 
     if (!uvm_context)
@@ -296,7 +293,7 @@ bool __uvm_check_lockable_order(uvm_lock_order_t lock_order, uvm_lock_flags_t fl
 
 bool __uvm_check_unlocked_order(uvm_lock_order_t lock_order)
 {
-    uvm_thread_context_t *uvm_context = uvm_thread_context();
+    uvm_thread_context_lock_t *uvm_context = uvm_thread_context_lock_get();
     if (!uvm_context)
         return true;
 
@@ -315,7 +312,7 @@ bool __uvm_check_unlocked_order(uvm_lock_order_t lock_order)
     return true;
 }
 
-bool __uvm_check_all_unlocked(uvm_thread_context_t *uvm_context)
+bool __uvm_check_all_unlocked(uvm_thread_context_lock_t *uvm_context)
 {
     uvm_lock_order_t lock_order;
     NvU32 still_locked_count;
@@ -332,7 +329,7 @@ bool __uvm_check_all_unlocked(uvm_thread_context_t *uvm_context)
     for_each_set_bit(lock_order, uvm_context->acquired_lock_orders, UVM_LOCK_ORDER_COUNT) {
         UVM_ERR_PRINT(" Lock %s, instance 0x%llx\n",
                       uvm_lock_order_to_string(lock_order),
-                      (NvU64)uvm_context->acquired_locks[lock_order]);
+                      (NvU64)uvm_context->acquired[lock_order]);
     }
 
     return false;
@@ -340,7 +337,7 @@ bool __uvm_check_all_unlocked(uvm_thread_context_t *uvm_context)
 
 bool __uvm_thread_check_all_unlocked()
 {
-    return __uvm_check_all_unlocked(uvm_thread_context());
+    return __uvm_check_all_unlocked(uvm_thread_context_lock_get());
 }
 
 NV_STATUS uvm_bit_locks_init(uvm_bit_locks_t *bit_locks, size_t count, uvm_lock_order_t lock_order)
